use chrono::offset::Utc;
use std::collections::HashMap;
use tauri::State;
use tokio::sync::Mutex;
use sqlx::sqlite::SqliteConnection as Connection;

use crate::types::*;

pub mod app_state;
pub mod data_door;

use app_state::AppState;

struct Id {
	id: i64,
}

fn parse_inventory(data: HashMap<String, String>) -> Result<Inventory, ()> {
	let a = data.get("cafe-inventory-acc").ok_or(())?;
	let acc: Account = Account::try_from(a.as_ref())?;
	let mut vouchers = Vec::new();
	for s in ["aldi", "dm", "lidl", "rewe", "tegut"] {
		let Ok(store) = s.try_into() else {
			println!("Did not find '{}' in inventory data.", s);
			continue;
		};
		match data.get(&format!("cafe-inventory-{}", s)) {
			None => (),
			Some(c) => {
				let c = if c == "" { "0" } else { c };
				let Ok(count): Result<i64, _> = c.parse() else {
					println!("Invalid count '{}' for '{}' in inventory data.", c, s);
					continue;
				};
				let value = 50_00;
				let voucher = VoucherType { store, value };
				let v = (voucher, count);
				vouchers.push(v);
			}
		}
	}
	unimplemented!()
}

pub async fn voucher_id(v: VoucherType, db: &mut Connection) -> Result<i64, ()> {
	sqlx::query!(
		"INSERT OR IGNORE INTO voucher_type(store, value) VALUES (?1, ?2)",
		v.store,
		v.value,
	)
	.execute(&mut *db)
	.await
	.map_err(|e| println!("{:?}", e))?;
	let id = sqlx::query_as!(
		Id,
		"SELECT id FROM voucher_type WHERE store = ?1 AND value = ?2",
		v.store,
		v.value,
	)
	.fetch_one(db)
	.await
	.map_err(|e| println!("{:?}", e))?
	.id;
	Ok(id)
}

#[tauri::command]
pub async fn inventory(
	data: HashMap<String, String>,
	state: State<'_, Mutex<AppState>>,
) -> Result<(), ()> {
	println!("{:?}", data);
	let now = Utc::now().timestamp();
	let mut state = state.lock().await;
	let inv = parse_inventory(data)?;
	sqlx::query!(
		"INSERT INTO inventory(acc, cash, timestamp) VALUES (?1, ?2, ?3)",
		inv.acc,
		inv.cash,
		now,
	)
	.execute(&mut state.db)
	.await
	.map_err(|e| println!("{:?}", e))?;
	let inventory = sqlx::query_as!(
		Id,
		"SELECT id FROM inventory WHERE acc = ?1 AND timestamp = ?2",
		inv.acc,
		now,
	)
	.fetch_one(&mut state.db)
	.await
	.map_err(|e| println!("{:?}", e))?
	.id;
	for (v, count) in inv.vouchers {
		let voucher = voucher_id(v, &mut state.db).await?;
		sqlx::query!(
			"INSERT INTO voucher_inventory VALUES (?1, ?2, ?3)",
			inventory,
			voucher,
			count,
		)
		.execute(&mut state.db)
		.await
		.map_err(|e| println!("{:?}", e))?;
	}
	Ok(())
}

#[tauri::command]
pub async fn swap(
	store: Store,
	acc: i64,
	state: State<'_, Mutex<AppState>>,
) -> Result<(), ()> {
	let timestamp = Utc::now().timestamp();
	let mut state = state.lock().await;
	let id = i64::from_ne_bytes(state.id.to_ne_bytes());
	let value = 50_00;
	let voucher = VoucherType{ store, value };
	let voucher_type = voucher_id(voucher, &mut state.db).await?;
	sqlx::query!(
		"INSERT INTO swap(acc, voucher, storno, timestamp) VALUES (?1, ?2, ?3, ?4)",
		acc,
		voucher_type,
		false,
		timestamp,
	)
	.execute(&mut state.db)
	.await
	.map_err(|e| println!("{:?}", e))?;
	Ok(())
}
